# -*- coding: utf-8 -*-
"""RE pattern objects for detecting and splitting Pinyin.

Splitting pinyin into syllables is not as simple as looking for the maximum
length matches using valid syllables. Instead, lookahead/lookbehind assertions
must be used to validate possible matches. For syllables, the rough approach
used is:
    * Get the longest valid syllable.
    * If it ends in a consonant make sure it's not followed directly by a
        vowel (a hyphen or apostrophe doesn't count).
    * If the above didn't match, repeat for the next longest valid match.

Lookahead/lookbehind assertions are used to ensure that hyphens and
apostrophes are only included in words if used correctly. This helps to weed
out non-Pinyin strings.

"""

from __future__ import unicode_literals
from string import whitespace


vowels = (
    'aɑeiouüvAEIOUÜV'
    'āɑ̄ēīōūǖĀĒĪŌŪǕ'
    'áɑ́éíóúǘÁÉÍÓÚǗ'
    'ǎɑ̌ěǐǒǔǚǍĚǏǑǓǙ'
    'àɑ̀èìòùǜÀÈÌÒÙǛ'
)
consonants = 'bpmfdtnlgkhjqxzcsrzcswyBPMFDTNLGKHJQXZCSRZCSWY'
marks = "·012345:-'"
non_stops = """"#$%&'()*+,-/:;<=>@[\]^_`{|}~"""
stops = '.!?'
printable = vowels + consonants + marks[:-3] + whitespace + stops + non_stops

_a = 'a\u0101\u00E0\u00E1\u01CE'
_e = 'e\u0113\u00E9\u011B\u00E8'
_i = 'i\u012B\u00ED\u01D0\u00EC'
_o = 'o\u014D\u00F3\u01D2\u00F2'
_u = 'u\u016B\u00FA\u01D4\u00F9'
_v = 'v\u00FC\u01D6\u01D8\u01DA\u01DC'

# This is the end-of-syllable-consonant lookahead assertion.
_consonant_end = '(?![%(a)s%(e)s%(i)s%(o)s%(u)s%(v)s]|u:)' % {
    'a': _a, 'e': _e, 'i': _i, 'o': _o, 'u': _u, 'v': _v
}

syl = syllable = (
    '(?:\u00B7|\u2027)?'
    '(?:'
    '(?:(?:[zcs]h|[gkh])u[%(a)s]ng%(consonant_end)s)|'
    '(?:[jqx]i[%(o)s]ng%(consonant_end)s)|'
    '(?:[nljqx]i[%(a)s]ng%(consonant_end)s)|'
    '(?:(?:[zcs]h?|[dtnlgkhrjqxy])u[%(a)s]n%(consonant_end)s)|'
    '(?:(?:[zcs]h|[gkh])u[%(a)s]i)|'
    '(?:(?:[zc]h?|[rdtnlgkhsy])[%(o)s]ng%(consonant_end)s)|'
    '(?:(?:[zcs]h?|[rbpmfdtnlgkhw])?[%(e)s]ng%(consonant_end)s)|'
    '(?:(?:[zcs]h?|[rbpmfdtnlgkhwy])?[%(a)s]ng%(consonant_end)s)|'
    '(?:[bpmdtnljqxy][%(i)s]ng%(consonant_end)s)|'
    '(?:[bpmdtnljqx]i[%(a)s]n%(consonant_end)s)|'
    '(?:[bpmdtnljqx]i[%(a)s]o)|'
    '(?:[nl](?:v|u:|\u00FC)[%(e)s])|'
    '(?:[nl](?:[%(v)s]|u:))|'
    '(?:[jqxy]u[%(e)s])|'
    '(?:[bpmnljqxy][%(i)s]n%(consonant_end)s)|'
    '(?:[mdnljqx]i[%(u)s])|'
    '(?:[bpmdtnljqx]i[%(e)s])|'
    '(?:[dljqx]i[%(a)s])|'
    '(?:(?:[zcs]h?|[rdtnlgkhxqjy])[%(u)s]n%(consonant_end)s)|'
    '(?:(?:[zcs]h?|[rdtgkh])u[%(i)s])|'
    '(?:(?:[zcs]h?|[rdtnlgkh])u[%(o)s])|'
    '(?:(?:[zcs]h|[rgkh])u[%(a)s])|'
    '(?:(?:[zcs]h?|[rbpmfdngkhw])?[%(e)s]n%(consonant_end)s)|'
    '(?:(?:[zcs]h?|[rbpmfdtnlgkhwy])?[%(a)s]n%(consonant_end)s)|'
    '(?:(?:[zcs]h?|[rpmfdtnlgkhy])?[%(o)s]u)|'
    '(?:(?:[zcs]h?|[rbpmdtnlgkhy])?[%(a)s]o)|'
    '(?:(?:[zs]h|[bpmfdtnlgkhwz])?[%(e)s]i)|'
    '(?:(?:[zcs]h?|[bpmdtnlgkhw])?[%(a)s]i)|'
    '(?:(?:[zcs]h?|[rjqxybpmdtnl])[%(i)s])|'
    '(?:(?:[zcs]h?|[rwbpmfdtnlgkhjqxwy])[%(u)s])|'
    '(?:(?:[zcs]h?|[rmdtnlgkhy])?[%(e)s])|'
    '(?:[bpmfwyl]?[%(o)s])|'
    '(?:(?:[zcs]h|[bpmfdtnlgkhzcswy])?[%(a)s])'
    ')(?:r%(consonant_end)s)?[0-5]?'
) % {
    'consonant_end': _consonant_end, 'a': _a, 'e': _e, 'i': _i,
    'o': _o, 'u': _u, 'v': _v
}

word = (
    """(?:%(as)s(?:-(?=%(as)s)|'(?=[%(a)s%(e)s%(o)s])(?=%(as)s))?[0-9]*)+"""
) % {'as': syllable, 'a': _a, 'e': _e, 'o': _o}

sent = sentence = (
    """(?:%(word)s|[%(non_stops)s\s])+[.!?]['"\]\}\)]*"""
) % {'word': word, 'non_stops': non_stops}



# -*- coding: utf-8 -*-
"""Constants for working with Chinese characters."""

from __future__ import unicode_literals
import sys

characters = (
    '\u4E00-\u9FFF'  # CJK Unified Ideographs
    '\u3400-\u4DBF'  # CJK Unified Ideographs Extension A
    '\uF900-\uFAFF'  # CJK Compatibility Ideographs
    '\u9FA6-\u9FCB'  # Extension to the URO
)
if sys.maxunicode > 0xFFFF:
    characters += (
        '\U00020000-\U0002A6DF'  # CJK Unified Ideographs Extension B
        '\U0002A700-\U0002B73F'  # CJK Unified Ideographs Extension C
        '\U0002B740-\U0002B81F'  # CJK Unified Ideographs Extension D
        '\U0002F800-\U0002FA1F'  # CJK Compatibility Ideographs Supplement
    )

radicals = (
    '\u2F00-\u2FD5'  # Kangxi Radicals
    '\u2E80-\u2EF3'  # CJK Radicals Supplement
)

non_stops = (
    '\uFF02-\uFF0D'                   # Fullwidth ASCII variants
    '\uFF0F'                          # Fullwidth ASCII variants
    '\uFF1A-\uFF1E'                   # Fullwidth ASCII variants
    '\uFF20'                          # Fullwidth ASCII variants
    '\uFF3B-\uFF40'                   # Fullwidth ASCII variants
    '\uFF5B-\uFF60'                   # Fullwidth ASCII variants
    '\uFF62-\uFF64'                   # Halfwidth CJK punctuation
    '\u3000-\u3001'                   # CJK symbols and punctuation
    '\u3003'                          # CJK symbols and punctutaion
    '\u3008-\u3011'                   # CJK angle and corner brackets
    '\u3014-\u301F'                   # CJK brackets and symbols/punctuation
    '\u3030'                          # Other CJK symbols
    '\u303E-\u303F'                   # Special CJK indicators
    '\u2013-\u2014'                   # Dashes
    '\u2018-\u2019'                   # Quotation marks and apostrophe
    '\u201B-\u201F'                   # Quotation marks and apostrophe
    '\u2026-\u2027'                   # General punctuation
    '\uFE4F'                          # Overscores and underscores
)

stops = (
    '\uFF01'  # Fullwidth exclamation mark
    '\uFF1F'  # Fullwidth question mark
    '\uFF61'  # Halfwidth ideographic full stop
    '\u3002'  # Ideographic full stop
)

punctuation = non_stops + stops

# A sentence end is defined by a stop followed by zero or more
# container-closing marks (e.g. quotation or brackets).
_sentence_end = '[%(stops)s][」﹂”』’》）］｝〕〗〙〛〉】]*' % {'stops': stops}

sentence = '[%(characters)s%(radicals)s%(non_stops)s]*%(sentence_end)s' \
    % {'characters': characters, 'radicals': radicals, 'non_stops': non_stops,
        'sentence_end': _sentence_end}



"""Provides CC-CEDICT traditional and simplified character constants."""

from __future__ import unicode_literals

from .simplified import CHARACTERS as simplified
from .traditional import CHARACTERS as traditional



from setuptools import setup
import sys

if sys.version_info < (3, ):
    import codecs
    enc_open = codecs.open
else:
    enc_open = open


with enc_open('README.rst', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='zhon',
    version='1.0',
    author='Thomas Roten',
    author_email='thomas@roten.us',
    url='https://github.com/tsroten/zhon',
    description=('Zhon provides constants used in Chinese text processing.'),
    long_description=long_description,
    packages=['zhon', 'zhon.cedict'],
    keywords=('chinese mandarin segmentation tokenization punctuation hanzi '
              'unicode radicals han cjk cedict cc-cedict traditional '
              'simplified characters pinyin zhuyin'),
    classifiers=[
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Text Processing :: Linguistic',
    ],
    platforms='Any',
    test_suite='tests',
)
